#!/usr/bin/perl -w
#
# install or remove the kernel module acl8112
#   also create or delete the special files in /dev
#

#-----------------------------------------------------------------------
# configuration parameters
#
# to indicate the number of ACL8112 cards (up to 4)
# uncomment one of the next 4 lines 
#@cards = qw(A);
@cards = qw(A B);
#@cards = qw(A B C);
#@cards = qw(A B C D);
#
# the following examples are for two cards, A & B:

# io ports:
%io = (A => 0x220, B => 0x230);

# card type (HG: 'H'=72, DG: 'D'=68, PG: 'P'=80) 
%ctp = (A => 72, B => 72);

# analog input bipolar (1) or unipolar (0):
%bp = (A => 0, B => 1);

# analog output voltage range
%aor = (A => 5, B => 10);

# analog input voltage range (PG card type only)
#%air = (A => 5, B => 5);
#

# directory where acl8112.o is found
$acldir = "/usr/local/acl8112";

#-------------------------------------------------------------------------
#-------------- end of configuration parameters --------------------------
#-------------------------------------------------------------------------


$fil = "$acldir/acl8112.o";
if(! -e $fil){
    die "$fil not found\n";
}

@devices = qw(ai0 ai1 ai2 ai3 ai4 ai5 ai6 ai7 ai8 ai9 ai10
	      ai11 ai12 ai13 ai14 ai15 ao0 ao1 di do);
%cardbits = (A => 0, B => 0x40, C => 0x80, D => 0xb0); 


push @par, prep("io");
push @par, prep("bp");
push @par, prep("air");
push @par, prep("aor");
push @par, prep("ctp");
$rc = system "insmod",$fil,@par,@ARGV;
if($rc != 0){
    $par = join " ",@par,@ARGV;
    die "insmod failed with parameters:\n$par\n";
}
%io=%bp=%air=%aor=%ctp=(); # (to avoid warnings)
%io=%bp=%air=%aor=%ctp=(); # (to avoid warnings)

open(INP,"</proc/devices");
while(<INP>){
    ($major,$name) = split(" ");
    last if $name eq "acl8112";
}
close INP;

# testing nodes in /dev for duplicate major number
$cfl = major_conflicts();
if($cfl){
    $rc = system "rmmod acl8112";
    print STDERR "$!" unless $rc==0;
    exit -1;
}

foreach $card(@cards){
    for($typ=0; $typ<=$#devices; $typ++){
	$minor = $typ | $cardbits{$card};
	$file = "/dev/acl".$card.$devices[$typ];
	unlink $file if -e $file;
	$rc = system "mknod",$file,"c",$major,$minor;
	print STDERR "$!" unless $rc==0;
	chmod 0666,$file;
	print STDERR "$!" unless $rc==0;
    }
}

sub major_conflicts{
    my $cfl = 0;
    opendir(DEV,"/dev");
    while($dev = readdir(DEV)){
	if(-c "/dev/$dev"){
	    @stat = stat("/dev/$dev");
	    $rdev = $stat[6];
	    if($major == ($rdev>>8)){
		if($dev =~ m/^acl/){
		    unlink "/dev/$dev";
		}
		else{
		    warn "acl8112/insmod.pl:\nMajor number conflict with $dev.\nPlease delete $dev!\n";
		    $cfl++;
		}
	    }
	}
    }
    return $cfl;
}


sub prep{ # prepare insmod parameter string
    my $nam = shift;
    my ($hp,$str,$len);

    eval "\$hp = \\%main::$nam"; 
    die $@ if $@;

    $len = scalar keys %$hp;
    if($len == 0){
	return "";
    }
    else{
	$str = "$nam=";
	foreach $card (qw(A B C D)){
	    if($hp->{$card}){
		$str .= $hp->{$card};
	    }
	    if($card ne "D"){
		$str .= ",";
	    }
	}
	return $str;
    }
}
